/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*jslint devel: true*/
/*global window, document, tizen*/

var svgBoard = null,
    content = null,
    shapeElements = null,
    shapeIndex = 0,
    shapeColor = "",
    shapeSizeTemp = 0,
    shapeSize = 0,
    svgShapeTemp = null,
    svgShape = null,
    svgShapeTag = "",
    svgRect = "<rect width='200' height='200' style='' />",
    svgRectRnd = "<rect rx='70' ry='70' width='200' height='200' style='' />",
    svgEllipse = "<ellipse cx='100' cy='100' rx='100' ry='100' style='' />",
    svgTriangle = "<polygon points='100,0 0,200 200,200' style='fill-rule:nonzero;' />",
    svgStar = "<polygon points='100,0 40,200 200,76 0,76 160,200' style='fill-rule:nonzero;' />",
    drawPath = {},
    timer = 0,
    touchDuration = 200,
    isLongTouch = 0,
    isPopup = 0;

/**
 * SVG drawing & editing feature
 */

function touchMoveShape(e) {
    var i = 0,
        j = 0,
        x = 0,
        y = 0,
        touches = e.changedTouches,
        touchesLength = touches.length,
        currentDrawPath = null,
        originalSize = 0,
        originalMatrix = "",
        newMatrix = "",
        matrixArray = null;

    e.target.style.opacity = 0.5;

    for (i = 0; i < touchesLength; i += 1) {
        currentDrawPath = drawPath[touches[i].identifier];

        if (currentDrawPath !== undefined) {
            originalMatrix = e.target.getAttribute("transform");
            matrixArray = originalMatrix.split(" ");
            originalSize = Number(matrixArray[3]);

            x = currentDrawPath.pageX - content.offsetLeft - (originalSize * 100);
            y = currentDrawPath.pageY - content.offsetTop - (originalSize * 100);

            matrixArray[4] = x;
            matrixArray[5] = y + ")";

            for (j = 0; j < matrixArray.length; j++) {
                newMatrix += matrixArray[j] + " ";
            }

            e.target.setAttribute("transform", newMatrix);

            drawPath[touches[i].identifier] = touches[i];
        }
    }
    e.preventDefault();
}

function touchEndShape(e) {
    var i = 0,
        x = 0,
        y = 0,
        touch = e.changedTouches[0],
        originalSize = 0,
        originalMatrix = "",
        newMatrix = "",
        matrixArray = null;

    originalMatrix = e.target.getAttribute("transform");
    matrixArray = originalMatrix.split(" ");
    originalSize = Number(matrixArray[3]);

    x = touch.pageX - content.offsetLeft - (originalSize * 100);
    y = touch.pageY - content.offsetTop - (originalSize * 100);

    e.target.style.opacity = 1;

    matrixArray[4] = x;
    matrixArray[5] = y + ")";

    for (i = 0; i < matrixArray.length; i++) {
        newMatrix += matrixArray[i] + " ";
    }

    e.target.setAttribute("transform", newMatrix);

    delete drawPath[touch.identifier];
}

function shapeEvents(id) {
    var svgShape = document.querySelector('#' + id);

    svgShape.addEventListener('touchmove', touchMoveShape, false);
    svgShape.addEventListener('touchend', touchEndShape, false);
}

function innerHtmlSvg(boardId, shapeId, shapeTag, shapeRatio, traslateX, traslateY) {
    var tempDiv = document.createElement('div'), // Create a dummy tempDiv
        svgFragment = "",
        xySize = 0,
        xyTag = "",
        colorTag = "";

    colorTag = "style='fill:" + shapeColor + ";";
    shapeTag = shapeTag.replace("style='", colorTag);

    xySize = 0.25 * shapeRatio;
    xyTag = "transform='matrix(" + xySize + " 0 0 " + xySize + " " + traslateX + " " + traslateY + ")' id='" + shapeId + "'/>";
    shapeTag = shapeTag.replace(" />", xyTag);

    svgFragment = '<svg>' + shapeTag + '</svg>'; // Wrap the svg string to a svg object
    tempDiv.innerHTML = svgFragment; // Add all svg to the tempDiv
    Array.prototype.slice.call(tempDiv.childNodes[0].childNodes).forEach(function(el) {
        document.querySelector('#' + boardId).appendChild(el);
    }); // Splice the children of the tempDiv's SVG to the actual body's SVG

    shapeIndex++;

    if (boardId === "svg-board") {
        shapeEvents(shapeId);
    }
}

/**
 * Board touch events feature
 */

function longtouchHandler() {
    isLongTouch = 1;
}

function touchStartHandler(e) {
    var touch = e.changedTouches[0];

    drawPath[touch.identifier] = touch;

    timer = setTimeout(longtouchHandler, touchDuration);
}

function touchEndHandler(e) {
    var touch = e.changedTouches[0],
        x = 0,
        y = 0;

    // Adding a new shape in the SVG board
    if (!isLongTouch) {
        x = touch.pageX - content.offsetLeft - (0.25 * shapeSize * 100);
        y = touch.pageY - content.offsetTop - (0.25 * shapeSize * 100);

        innerHtmlSvg("svg-board", "shape" + shapeIndex, svgShapeTag, shapeSize, x, y);
        delete drawPath[touch.identifier];
    }

    // Stops short touches from firing the event
    isLongTouch = 0;
    if (timer) {
        clearTimeout(timer);
    }
}

/**
 * Pop-up feature
 */

function showShapePopup() {
    var i = 0;

    // Show the last selected shape and size (not temp)
    for (i = 0; i < shapeElements.length; i++) {
        shapeElements[i].style.opacity = 0.3;
    }
    document.querySelector('#' + svgShape).style.opacity = 1.0;

    document.querySelector('#size-input').value = shapeSize;
    document.querySelector('#size-result').innerHTML = shapeSize;

    document.querySelector('#option-popup').style.visibility = 'visible';
    document.querySelector('#option-popup-bg').style.visibility = 'visible';

    isPopup = 1;
}

function hideShapePopup() {
    document.querySelector('#option-popup').style.visibility = 'hidden';
    document.querySelector('#option-popup-bg').style.visibility = 'hidden';

    isPopup = 0;
}

/**
 * Color/Shape Selector
 */

function changeColor(e) {
    shapeColor = e.target.value;
}

function changeSize(e) {
    shapeSizeTemp = e.target.value;
    document.querySelector('#size-result').innerHTML = shapeSizeTemp;
}

function changeShape() {
    var shapeInput = document.querySelector('#shape-input-svg');

    shapeSize = shapeSizeTemp;
    svgShape = svgShapeTemp;

    switch (svgShape) {
        case "select-rect":
            svgShapeTag = svgRect;
            break;
        case "select-rectrnd":
            svgShapeTag = svgRectRnd;
            break;
        case "select-ellipse":
            svgShapeTag = svgEllipse;
            break;
        case "select-triangle":
            svgShapeTag = svgTriangle;
            break;
        case "select-star":
            svgShapeTag = svgStar;
            break;
    }

    shapeInput.removeChild(shapeInput.lastChild);
    innerHtmlSvg("shape-input-svg", "select-shape", svgShapeTag, 4, 0, 0);

    hideShapePopup();
}

function clickShape(event) {
    var shapeClicked = event.target,
        i = 0;

    for (i = 0; i < shapeElements.length; i++) {
        shapeElements[i].style.opacity = 0.3;
    }
    shapeClicked.style.opacity = 1.0;

    svgShapeTemp = shapeClicked.id;
}

/**
 * Board Cleaner
 */

function clearSvgBoard() {
    while (svgBoard.lastChild) {
        svgBoard.removeChild(svgBoard.lastChild);
    }
}

function bindEvents() {
    var i = 0;

    // SVG board click events
    svgBoard.addEventListener('touchstart', touchStartHandler, false);
    svgBoard.addEventListener('touchend', touchEndHandler, false);

    // Additional events
    document.querySelector('#color-input').addEventListener('change', changeColor, false);
    document.querySelector('#size-input').addEventListener('change', changeSize, false);
    document.querySelector('#clear-input').addEventListener('click', clearSvgBoard, false);
    document.querySelector('#shape-input').addEventListener('click', showShapePopup, false);
    document.querySelector('#shape-cancel-btn').addEventListener('click', hideShapePopup, false);
    document.querySelector('#shape-set-btn').addEventListener('click', changeShape, false);

    for (i = 0; i < shapeElements.length; i++) {
        shapeElements[i].addEventListener('click', clickShape);
    }
}


function init() {
    svgShape = "select-rect";
    shapeSize = 1;
    svgShapeTag = svgRect;
    svgShapeTemp = svgShape;
    shapeSizeTemp = shapeSize;
    shapeElements = document.querySelectorAll('.shape-element');
    content = document.querySelector('.content');

    // Set the SVG Board size
    svgBoard = document.querySelector('#svg-board');
    svgBoard.width = content.clientWidth;
    svgBoard.height = content.clientHeight;

    innerHtmlSvg("shape-input-svg", "select-shape", svgShapeTag, 4, 0, 0);

    bindEvents();

    // Add eventListener for tizenhwkey
    window.addEventListener('tizenhwkey', function onTizenHwKey(e) {
        if (e.keyName === 'back') {
            if (isPopup === 1) {
                hideShapePopup();
            } else {
                try {
                    tizen.application.getCurrentApplication().exit();
                } catch (err) {
                    console.error('Error: ', err);
                }
            }
        }
    });
}

window.onload = function() {
    init();
}